---------------------------------------------------------------------------------
-- Model methods
-----------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- Channel methods
-----------------------------------------------------------------------------------

function changeChannel(ix)
	if (ix ~= -1) then
		downloadAppStop()
		canvas:flush()
		channel.change(ix)
	else
		log( TRACE, 'no channel')
	end
end

function changeFactor( factor )
	local ch = channels.next( channel.current(), factor )
	log( TRACE, 'changeFactor: ch='..ch )
	changeChannel(ch,isVirtual)
end

function nextAudio()
	channel.nextAudio()
end

function nextSubtitle()
	channel.nextSubtitle()
end

---------------------------------------------------------------------------------
-- Audio methods
-----------------------------------------------------------------------------------

function toggleMute()
	player.toggleMute()
end

function volumenUp()
	player.volumeUp()
end

function volumenDown()
	player.volumeDown()
end

---------------------------------------------------------------------------------
-- Loop methods
-----------------------------------------------------------------------------------

function exitLoop()
	--	Cancel scan, if necesary
	mainWindow.stop()
end

--------------------------------------------------------------------------------------------------
-- Zapper callbacks
--------------------------------------------------------------------------------------------------

function zapperChannelChanged( chID )
	log( TRACE, "Zapper channel info changed: chID=" .. chID )
end

function zapperCurrentShowBlocked()
	log( TRACE, "Current show blocked" )
end

function zapperUpdateReceived( updateID )
	log( TRACE, "Zapper update received: updateID=" .. updateID )
end

function zapperUpdateDownloadProgress( step, total )
	local value = (step * 100) / total
	log( TRACE, "Download update progress: step=" .. step .. " total=" .. total .. " value=" .. value )
end

function zapperApplicationLayerEnabled( isEnabled )
	log( TRACE, "Applications running:" .. isEnabled )
	downloadAppStop()
	canvas:flush()
end

function zapperApplicationDownloadProgress( step, total )
	percentage = math.floor( step * 100 / total )
	log( TRACE, "Download application: step=" .. step .. " total=" .. total .. " progress=" .. percentage )
	downloadAppUpdate( percentage )
	canvas:flush()
end

--------------------------------------------------------------------------------------------------
-- Timers
--------------------------------------------------------------------------------------------------

Timers = {
	download = {id=1, timeout=5000, action=downloadAppStop}
}

function OnTimerEvent( tID )
	logMethod( TRACE, "OnTimerEvent", "id=" .. tID )
	for _,timer in pairs(Timers) do
		if (tID == timer['id']) and timer.action then
			timer.action()
			break
		end
	end
	canvas:flush()
end
