----------------------------------------------------------------------------------
-- Scan Window
-----------------------------------------------------------------------------------

ScanWindow = wndNew( 'Scan', nil, 160, 464 )

function ScanWindow.actions.onCreate( win )
	log( TRACE, 'ScanWindow::onCreate' )

	win.variables = {
		inScan = false,
		networks = 0,
		progress = 0,
		currentNetwork = '',
		channels = 0,
	}

	--	Create scan progress bar
	win.variables.wdt = pbNew(
		'scanProgressBar',
		win.x, win.y,
		'FondoScan.png',
		makeProgressLabel(0),
		14, 15, 20, 15, 34, 368, 14
	)
	wndAddWidget( win, win.variables.wdt )

	--	Setup keys
	win.keys.Release = {
		  [KEY_EXIT]   = 'onCancelScan'
		, [KEY_ESCAPE] = 'onCancelScan'
	}
end

function ScanWindow.actions.onCancelScan( win )
	wdtSetVisible( ScanWindow, false )
end

-- Scan methods
function startScan( parent, onScanClosed )
	logMethod( TRACE, 'Scan::startScan', ScanWindow.variables.inScan )
	if not ScanWindow.variables.inScan then
		ScanWindow.variables.inScan = true
		
		--	Hide the download application dialog
		downloadAppStop()

		if onScanClosed == nil then
			onScanClosed = emptyMethod
		end
		wdtSetParent( ScanWindow, parent )
		wdtAddAction( ScanWindow, 'onClose', function() stopScan(true) onScanClosed() end )
		wdtSetVisible( ScanWindow, true )

		tuner.startScan()
	end
end

function stopScan( updateChannel )
	logMethod( TRACE, 'Scan::stopScan', ScanWindow.variables.inScan )
	if ScanWindow.variables.inScan then
		ScanWindow.variables.inScan = false
		tuner.stopScan()
		runAction( ScanWindow, 'onCancelScan' )

		if (updateChannel and channels.count() > 0) then
			changeChannel( 0 )
			mainWindow.showBackground(0)
		end
	end
end

function updateScan()
	logMethod( TRACE, 'updateScan', 'progress: ' .. ScanWindow.variables.progress, 'networks: ' .. ScanWindow.variables.networks )
	local value = math.floor((ScanWindow.variables.progress * 100) / ScanWindow.variables.networks)
	pbUpdate( 
		ScanWindow.variables.wdt, 
		value,
		makeProgressLabel(value)
	)
	canvas:flush()
end

function makeProgressLabel( value )
	return 'Buscando canales... ' .. tostring(value) .. '% (' .. ScanWindow.variables.currentNetwork .. 'Mhz) - ' .. tostring(ScanWindow.variables.channels) .. ' encontrados'
end

-- Zapper callback
function zapperBeginScan( count )
	log( TRACE, "Zapper begin scan: count=" .. count )
	ScanWindow.variables.progress = 0
	ScanWindow.variables.channels = 0
	ScanWindow.variables.networks = count
	ScanWindow.variables.currentNetwork = ''
	updateScan()
end

function zapperScanNetwork( net )
	log( TRACE, "Zapper scan network: network=" .. net )
	ScanWindow.variables.progress = ScanWindow.variables.progress + 1
	ScanWindow.variables.currentNetwork = net
	updateScan()
end

function zapperProgramFound( ch )
	log( TRACE, "Zapper program found!" )
	ScanWindow.variables.channels = ScanWindow.variables.channels + 1
	updateScan()
end

function zapperEndScan()
	log( TRACE, "Zapper end scan!" )
	stopScan( true )
	canvas:flush()
	ScanWindow.variables.channels = 0
end

