--------------------------------------------------------------------------------------------------
-- Button methods
--------------------------------------------------------------------------------------------------

function btnAddAction( btn, action )
	wdtAddAction( btn, 'onSelected', action )
end

function btnAddShortcut( btn, key )
	wdtAddKey( btn, key, 'Press', 'onBtnPress' )
	wdtAddKey( btn, key, 'Release', 'onKeyRelease' )
end

function btnToggleSelected( btn )
	btn.selected = not btn.selected
	btnRedraw( btn, 'down' )
end

function btnSetSelected( btn, sel, state )
	logMethod( TRACE, 'btnSetSelected', btn['name'], sel, state )
	btn.selected = sel
	btnRedraw( btn, state )
end

function btnReleaseOnSelected( btn, release )
	btn.releaseOnSelected = release
end

function btnNew( name, x, y, upBtn, downBtn, upBtnSel, downBtnSel )
	local btn = wdtNew( name, nil, x, y, 'Button' )

	-- Default init
	btn.selected = false

	-- Images
	btn.images = {}
	btn.images[true] = {}
	btn.images[false] = {}
	btn.images[false].up = upBtn
	btn.images[false].down = downBtn
	btn.images[true].up = upBtnSel
	btn.images[true].down = downBtnSel

	-- Fix if sel is empty
	if (btn.images[true].up == nil) then
		btn.images[true].up = upBtn
	end

	if (btn.images[true].down == nil) then
		btn.images[true].down = downBtn
	end

	--	w,h
	btn.w,btn.h = canvas:imageSize( btn.images[false].down )

	-- Button regions
	wdtAddButtonRegion( btn, 1, true, btn.x, btn.y, btn.w, btn.h )
	wdtAddButtonRegion( btn, 1, false, btn.x, btn.y, btn.w, btn.h )

	-- Actions
	btn.actions.onShow = btnOnShow
	btn.actions.onHide = btnOnHide
	btn.actions.onCreate = emptyMethod
	btn.actions.onKeyRelease = btnOnKeyRelease
	btn.actions.onBtnPress = btnOnPress
	btn.actions.onBtnRelease = btnOnRelease

	btn.releaseOnSelected = true

	return btn
end

-- Implementation
function btnRedraw( btn, state )
	--btnOnHide( btn )
	btnDraw( btn, state )
end

function btnOnShow( btn )
	btnDraw( btn, 'up' )
end

function btnDraw( btn, state )
	logMethod( TRACE, 'btnDraw', btn['name'], state )
	local img = btn.images[btn.selected][state]
	canvas:drawImage( img, btn.x, btn.y )
end

function btnOnHide( btn )
	logMethod( TRACE, 'btnOnHide', btn['name'] )
	canvas:clear( btn.x, btn.y, btn.w, btn.h )
end

function btnOnPress( btn )
	logMethod( TRACE, 'btnOnPress', btn['name'] );
	btnRedraw( btn, 'down' )
end

function btnOnRelease( btn, overBtn )
	logMethod( TRACE, 'btnOnRelease', btn['name'], overBtn );
	local redraw = true
	if (overBtn) then
		runAction( btn, 'onSelected' )
		redraw = btn.releaseOnSelected
	end
	if (redraw) then
		btnRedraw( btn, 'up' )
	end
end

function btnOnKeyRelease( btn )
	btnOnRelease( btn, true )
end
