--------------------------------------
-- Widget Slider
--------------------------------------

-- 0 <= percentage <= 100
function slUpdate( sl, percentage )
	logMethod( TRACE, 'slUpdate', sl['name'], percentage )
	sl.slider.pos.x = sl.slider.pos.minX + (percentage * (sl.slider.pos.maxX - sl.slider.pos.minX) / 100.0);
	if (sl.visible) then
		slOnShow( sl )
	end
end

function slSliderImage( sl, img )
	sl.slider.img = img
	if (sl.visible) then
		slOnShow( sl )
	end
end

-- 0 <= percentage <= 100 || percentage == nil
function slNew( name, x, y, sliderBarImg, sliderImg, percentage )

	local sl = wdtNew( name, nil, x, y, 'Slider' )
	sl.background = bgNew( name .. '_bg', x, y, sliderBarImg )

	slSetupSliderImage( sl, sliderImg )
	if (percentage ~= nil) then
		slUpdate( sl, percentage )
	end

	local sw, sh = canvas:imageSize( sl.slider.img )
	wdtAddButtonRegion( sl, 1, true, sl.slider.pos.minX, sl.slider.pos.y, sl.slider.pos.maxX-sl.slider.pos.minX+sw, sh )
	wdtAddButtonRegion( sl, 1, false, sl.slider.pos.minX, sl.slider.pos.y, sl.slider.pos.maxX-sl.slider.pos.minX+sw, sh )

	sl.actions.onShow = slOnShow
	sl.actions.onHide = slOnHide
	sl.actions.onBtnRelease = slOnButtonRelease
	sl.actions.onCreate = emptyMethod

	return sl
end

-- Implementation

function slOnShow( sl )
	logMethod( TRACE, 'slOnShow', sl['name'] )
	bgOnShow( sl.background )
	slDrawSlider( sl )
end

function slOnHide( sl )
	logMethod( TRACE, 'slOnHide', sl['name'] )
	bgOnHide( sl.background )
end

function slOnButtonRelease( sl, overSlider, x, y )
	logMethod( TRACE, 'slOnButton', sl['name'], overSlider )
	if (overSlider) then
		local sw, sh = canvas:imageSize( sl.slider.img )
		sl.slider.pos.x = x - ( sw / 2 )
		if (sl.slider.pos.x > sl.slider.pos.maxX) then
			sl.slider.pos.x = sl.slider.pos.maxX
		elseif (sl.slider.pos.x < sl.slider.pos.minX) then
			sl.slider.pos.x = sl.slider.pos.minX
		end
		slOnShow( sl )
		runAction( sl, 'onSliderPosChanged', (sl.slider.pos.x - sl.slider.pos.minX) * 100 / (sl.slider.pos.maxX - sl.slider.pos.minX) )
	end
end

function slDrawSlider( sl )
	logMethod( TRACE, 'slDrawSlider', sl['name'], sl.slider.pos.x, sl.slider.pos.y )
	canvas:drawImage( sl.slider.img, sl.slider.pos.x, sl.slider.pos.y )
end

function slSetupSliderImage( sl, img )
	local bw,bh = bgSize( sl.background )
	local sw,sh = canvas:imageSize( img )

	sl.slider = {
		['img'] = img,

		['pos'] = {
			['x'] = sl.x,
			['minX'] = sl.x,
			['maxX'] = sl.x + bw - sw,
			['y'] = sl.y + ((bh + sh) / 2) - sh
		}
	}
end
