---------------------------------------------------------------------------------
-- Variables
-----------------------------------------------------------------------------------

local CurrentWindow = nil
local KeysLocked = 0

---------------------------------------------------------------------------------
-- Key Locking ------------
-----------------------------------------------------------------------------------

function lockKeys()
	logMethod( TRACE, 'lockKeys', KeysLocked )
	KeysLocked = KeysLocked + 1
	if KeysLocked > 1 then
		mainWindow.lockKeys(1)
	end
end

function unlockKeys()
	logMethod( TRACE, 'unlockKeys', KeysLocked )
	KeysLocked = KeysLocked - 1
	if KeysLocked <= 1 then
		mainWindow.lockKeys(0)
	end
end

---------------------------------------------------------------------------------
-- Window methods
-----------------------------------------------------------------------------------

function wndCurrentWindow()
	return CurrentWindow
end

function setCurrentWindow( wnd )
	logMethod( TRACE, 'setCurrentWindow', wnd['name'] )
	CurrentWindow = wnd;
end

function wndNew( name, parent, x, y )
	win = wdtNew( name, parent, x, y, 'Window' )
	win.variables = {}
	win.widgets = {}
	win.w = 0
	win.h = 0
	win.actions.onShow = winOnShow
	win.actions.onHide = winOnHide
	win.actions.onKey = winOnKey
	win.actions.onButton = winOnButton
	return win
end

function wndResize( win, w, h )
	logMethod( TRACE, 'wndResize', win['name'], w, h )
	win.w = w
	win.h = h
end

function wndAddWidget( win, wdt )
	wdtCreate( wdt )
	wdtSetParent( wdt, win )
	table.insert( win.widgets, wdt )
end

function wndShowWidgets( win, show )
	for i=1, #win.widgets do
		wdtSetVisible( win.widgets[i], show )
	end
end

-- Implementation
function winOnShow( win )
	logMethod( TRACE, "winOnShow", win['name'] )
	lockKeys()
	setCurrentWindow( win )
	wndShowWidgets( win, true )
end

function winOnHide( win )
	logMethod( TRACE, "winOnHide", win['name'] )
	wndShowWidgets( win, false )
	runAction( win, 'onClose' )
	setCurrentWindow( win.parent )
	unlockKeys()
end

function winOnKey( win, key, isRelease )
	logMethod( TRACE, "winOnKey", win['name'], key, isRelease )
	for i=1, #win.widgets do
		if wdtHandleKey( win.widgets[i], key, isRelease ) then
			runAction( win.widgets[i], 'onKey', key, isRelease )
			return
		end
	end
	wdtOnKey( win, key, isRelease )
end

function winOnButton( win, button, isPress, x, y )
	logMethod( TRACE, "winOnButton", win['name'], button, isPress, x, y )

	if wdtPointInRect( win, x, y ) then
		for i=1, #win.widgets do
			if wdtHandleButton( win.widgets[i], button, isPress, x, y ) then
				runAction( win.widgets[i], 'onButton', button, isPress, x, y )
				return
			end
		end
	elseif isPress then
		wdtSetVisible( win, false )

		local cur=win.parent
		while not wdtPointInRect( cur, x, y ) do
			local tmp = cur
			cur = cur.parent
			wdtSetVisible( tmp, false )
		end
	end
end

function toogleFullScreen( btn )
	logMethod( TRACE, "menuOnFullScreen", mainWindow.isFullscreen() )
	mainWindow.fullscreen( not mainWindow.isFullscreen() );
end

